@@ -1,23 +1,26 @@
 use std::{collections::HashMap, env, fs};
 
-use tree_sitter::{LogType, Node, Parser, Point, Range, Tree};
+use tree_sitter::Parser;
 use tree_sitter_proc_macro::test_with_seed;
 
-use super::helpers::{
-    allocations,
-    edits::{get_random_edit, invert_edit},
-    fixtures::{fixtures_dir, get_language, get_test_language, SCRATCH_BASE_DIR},
-    new_seed,
-    random::Rand,
-    scope_sequence::ScopeSequence,
-    EDIT_COUNT, EXAMPLE_FILTER, ITERATION_COUNT, LANGUAGE_FILTER, LOG_ENABLED, LOG_GRAPH_ENABLED,
-    START_SEED,
-};
 use crate::{
+    fuzz::{
+        corpus_test::{
+            check_changed_ranges, check_consistent_sizes, get_parser, set_included_ranges,
+        },
+        edits::{get_random_edit, invert_edit},
+        flatten_tests, new_seed,
+        random::Rand,
+        EDIT_COUNT, EXAMPLE_FILTER, ITERATION_COUNT, LANGUAGE_FILTER, LOG_GRAPH_ENABLED,
+        START_SEED,
+    },
     generate,
     parse::perform_edit,
-    test::{parse_tests, print_diff, print_diff_key, strip_sexp_fields, TestEntry},
-    util,
+    test::{parse_tests, print_diff, print_diff_key, strip_sexp_fields},
+    tests::{
+        allocations,
+        helpers::fixtures::{fixtures_dir, get_language, get_test_language, SCRATCH_BASE_DIR},
+    },
 };
 
 #[test_with_seed(retry=10, seed=*START_SEED, seed_fn=new_seed)]
@@ -76,9 +79,10 @@ fn test_corpus_for_json(seed: usize) {
     test_language_corpus("json", seed, None, None);
 }
 
+#[ignore]
 #[test_with_seed(retry=10, seed=*START_SEED, seed_fn=new_seed)]
 fn test_corpus_for_php(seed: usize) {
-    test_language_corpus("php", seed, None, Some("php"));
+    test_language_corpus("php", seed, None, None);
 }
 
 #[test_with_seed(retry=10, seed=*START_SEED, seed_fn=new_seed)]
@@ -106,44 +110,45 @@ fn test_corpus_for_tsx(seed: usize) {
     test_language_corpus("typescript", seed, None, Some("tsx"));
 }
 
-fn test_language_corpus(
+pub fn test_language_corpus(
     language_name: &str,
     start_seed: usize,
     skipped: Option<&[&str]>,
-    subdir: Option<&str>,
+    language_dir: Option<&str>,
 ) {
-    let subdir = subdir.unwrap_or_default();
+    let language_dir = language_dir.unwrap_or_default();
 
     let grammars_dir = fixtures_dir().join("grammars");
     let error_corpus_dir = fixtures_dir().join("error_corpus");
     let template_corpus_dir = fixtures_dir().join("template_corpus");
-    let mut corpus_dir = grammars_dir.join(language_name).join(subdir).join("corpus");
-    if !corpus_dir.is_dir() {
-        corpus_dir = grammars_dir
-            .join(language_name)
-            .join(subdir)
-            .join("test")
-            .join("corpus");
-    }
+    let corpus_dir = grammars_dir.join(language_name).join("test").join("corpus");
+
+    println!("Testing {language_name} corpus @ {}", corpus_dir.display());
 
     let error_corpus_file = error_corpus_dir.join(format!("{language_name}_errors.txt"));
     let template_corpus_file = template_corpus_dir.join(format!("{language_name}_templates.txt"));
     let main_tests = parse_tests(&corpus_dir).unwrap();
     let error_tests = parse_tests(&error_corpus_file).unwrap_or_default();
     let template_tests = parse_tests(&template_corpus_file).unwrap_or_default();
-    let mut tests = flatten_tests(main_tests);
-    tests.extend(flatten_tests(error_tests));
-    tests.extend(flatten_tests(template_tests).into_iter().map(|mut t| {
-        t.template_delimiters = Some(("<%", "%>"));
-        t
-    }));
+    let mut tests = flatten_tests(main_tests, EXAMPLE_FILTER.as_ref());
+    tests.extend(flatten_tests(error_tests, EXAMPLE_FILTER.as_ref()));
+    tests.extend(
+        flatten_tests(template_tests, EXAMPLE_FILTER.as_ref())
+            .into_iter()
+            .map(|mut t| {
+                t.template_delimiters = Some(("<%", "%>"));
+                t
+            }),
+    );
+
+    tests.retain(|t| t.languages[0].is_empty() || t.languages.contains(&Box::from(language_dir)));
 
     let mut skipped = skipped.map(|x| x.iter().map(|x| (*x, 0)).collect::<HashMap<&str, usize>>());
 
-    let language_path = if subdir.is_empty() {
+    let language_path = if language_dir.is_empty() {
         language_name.to_string()
     } else {
-        format!("{language_name}/{subdir}")
+        format!("{language_name}/{language_dir}")
     };
     let language = get_language(&language_path);
     let mut failure_count = 0;
@@ -183,13 +188,14 @@ fn test_language_corpus(
             if actual_output != test.output {
                 println!("Incorrect initial parse for {test_name}");
                 print_diff_key();
-                print_diff(&actual_output, &test.output);
+                print_diff(&actual_output, &test.output, true);
                 println!();
                 return false;
             }
 
             true
-        });
+        })
+        .unwrap();
 
         if !passed {
             failure_count += 1;
@@ -270,7 +276,7 @@ fn test_language_corpus(
                 if actual_output != test.output {
                     println!("Incorrect parse for {test_name} - seed {seed}");
                     print_diff_key();
-                    print_diff(&actual_output, &test.output);
+                    print_diff(&actual_output, &test.output, true);
                     println!();
                     return false;
                 }
@@ -283,7 +289,7 @@ fn test_language_corpus(
                 }
 
                 true
-            });
+            }).unwrap();
 
             if !passed {
                 failure_count += 1;
@@ -371,7 +377,7 @@ fn test_feature_corpus_files() {
             let c_code = generate_result.unwrap().1;
             let language = get_test_language(language_name, &c_code, Some(&test_path));
             let test = parse_tests(&corpus_path).unwrap();
-            let tests = flatten_tests(test);
+            let tests = flatten_tests(test, EXAMPLE_FILTER.as_ref());
 
             if !tests.is_empty() {
                 eprintln!("test language: {language_name:?}");
@@ -393,11 +399,12 @@ fn test_feature_corpus_files() {
                         true
                     } else {
                         print_diff_key();
-                        print_diff(&actual_output, &test.output);
+                        print_diff(&actual_output, &test.output, true);
                         println!();
                         false
                     }
-                });
+                })
+                .unwrap();
 
                 if !passed {
                     failure_count += 1;
@@ -409,198 +416,3 @@ fn test_feature_corpus_files() {
 
     assert!(failure_count == 0, "{failure_count} corpus tests failed");
 }
-
-fn check_consistent_sizes(tree: &Tree, input: &[u8]) {
-    fn check(node: Node, line_offsets: &[usize]) {
-        let start_byte = node.start_byte();
-        let end_byte = node.end_byte();
-        let start_point = node.start_position();
-        let end_point = node.end_position();
-
-        assert!(start_byte <= end_byte);
-        assert!(start_point <= end_point);
-        assert_eq!(
-            start_byte,
-            line_offsets[start_point.row] + start_point.column
-        );
-        assert_eq!(end_byte, line_offsets[end_point.row] + end_point.column);
-
-        let mut last_child_end_byte = start_byte;
-        let mut last_child_end_point = start_point;
-        let mut some_child_has_changes = false;
-        let mut actual_named_child_count = 0;
-        for i in 0..node.child_count() {
-            let child = node.child(i).unwrap();
-            assert!(child.start_byte() >= last_child_end_byte);
-            assert!(child.start_position() >= last_child_end_point);
-            check(child, line_offsets);
-            if child.has_changes() {
-                some_child_has_changes = true;
-            }
-            if child.is_named() {
-                actual_named_child_count += 1;
-            }
-            last_child_end_byte = child.end_byte();
-            last_child_end_point = child.end_position();
-        }
-
-        assert_eq!(actual_named_child_count, node.named_child_count());
-
-        if node.child_count() > 0 {
-            assert!(end_byte >= last_child_end_byte);
-            assert!(end_point >= last_child_end_point);
-        }
-
-        if some_child_has_changes {
-            assert!(node.has_changes());
-        }
-    }
-
-    let mut line_offsets = vec![0];
-    for (i, c) in input.iter().enumerate() {
-        if *c == b'\n' {
-            line_offsets.push(i + 1);
-        }
-    }
-
-    check(tree.root_node(), &line_offsets);
-}
-
-fn check_changed_ranges(old_tree: &Tree, new_tree: &Tree, input: &[u8]) -> Result<(), String> {
-    let changed_ranges = old_tree.changed_ranges(new_tree).collect::<Vec<_>>();
-    let old_scope_sequence = ScopeSequence::new(old_tree);
-    let new_scope_sequence = ScopeSequence::new(new_tree);
-
-    let old_range = old_tree.root_node().range();
-    let new_range = new_tree.root_node().range();
-
-    let byte_range =
-        old_range.start_byte.min(new_range.start_byte)..old_range.end_byte.max(new_range.end_byte);
-    let point_range = old_range.start_point.min(new_range.start_point)
-        ..old_range.end_point.max(new_range.end_point);
-
-    for range in &changed_ranges {
-        if range.end_byte > byte_range.end || range.end_point > point_range.end {
-            return Err(format!(
-                "changed range extends outside of the old and new trees {range:?}",
-            ));
-        }
-    }
-
-    old_scope_sequence.check_changes(&new_scope_sequence, input, &changed_ranges)
-}
-
-fn set_included_ranges(parser: &mut Parser, input: &[u8], delimiters: Option<(&str, &str)>) {
-    if let Some((start, end)) = delimiters {
-        let mut ranges = Vec::new();
-        let mut ix = 0;
-        while ix < input.len() {
-            let Some(mut start_ix) = input[ix..]
-                .windows(2)
-                .position(|win| win == start.as_bytes())
-            else {
-                break;
-            };
-            start_ix += ix + start.len();
-            let end_ix = input[start_ix..]
-                .windows(2)
-                .position(|win| win == end.as_bytes())
-                .map_or(input.len(), |ix| start_ix + ix);
-            ix = end_ix;
-            ranges.push(Range {
-                start_byte: start_ix,
-                end_byte: end_ix,
-                start_point: point_for_offset(input, start_ix),
-                end_point: point_for_offset(input, end_ix),
-            });
-        }
-
-        parser.set_included_ranges(&ranges).unwrap();
-    } else {
-        parser.set_included_ranges(&[]).unwrap();
-    }
-}
-
-fn point_for_offset(text: &[u8], offset: usize) -> Point {
-    let mut point = Point::default();
-    for byte in &text[..offset] {
-        if *byte == b'\n' {
-            point.row += 1;
-            point.column = 0;
-        } else {
-            point.column += 1;
-        }
-    }
-    point
-}
-
-fn get_parser(session: &mut Option<util::LogSession>, log_filename: &str) -> Parser {
-    let mut parser = Parser::new();
-
-    if *LOG_ENABLED {
-        parser.set_logger(Some(Box::new(|log_type, msg| {
-            if log_type == LogType::Lex {
-                eprintln!("  {msg}");
-            } else {
-                eprintln!("{msg}");
-            }
-        })));
-    } else if *LOG_GRAPH_ENABLED {
-        *session = Some(util::log_graphs(&mut parser, log_filename, false).unwrap());
-    }
-
-    parser
-}
-
-struct FlattenedTest {
-    name: String,
-    input: Vec<u8>,
-    output: String,
-    has_fields: bool,
-    template_delimiters: Option<(&'static str, &'static str)>,
-}
-
-fn flatten_tests(test: TestEntry) -> Vec<FlattenedTest> {
-    fn helper(test: TestEntry, is_root: bool, prefix: &str, result: &mut Vec<FlattenedTest>) {
-        match test {
-            TestEntry::Example {
-                mut name,
-                input,
-                output,
-                has_fields,
-                ..
-            } => {
-                if !prefix.is_empty() {
-                    name.insert_str(0, " - ");
-                    name.insert_str(0, prefix);
-                }
-                if let Some(filter) = EXAMPLE_FILTER.as_ref() {
-                    if !name.contains(filter.as_str()) {
-                        return;
-                    }
-                }
-                result.push(FlattenedTest {
-                    name,
-                    input,
-                    output,
-                    has_fields,
-                    template_delimiters: None,
-                });
-            }
-            TestEntry::Group {
-                mut name, children, ..
-            } => {
-                if !is_root && !prefix.is_empty() {
-                    name.insert_str(0, " - ");
-                    name.insert_str(0, prefix);
-                }
-                for child in children {
-                    helper(child, false, &name, result);
-                }
-            }
-        }
-    }
-    let mut result = Vec::new();
-    helper(test, true, "", &mut result);
-    result
-}
